        .section .text,"ax",@progbits
#
# expects caller to pass the floating point value in xmm0
# computes FP(X), X is a floating point value,
# and this function returns the non-integer part as a floating
# point value in xmm0
#
        .type   .LFP, @function
        .align 16, 0x90
.LFP:
        pushq   %rbp                  # create the stack frame
        movq    %rsp, %rbp            # and reserve space for the
        pushfq                        # save backup copy of flags   (8)
        pushq   %rax                  #                             (8)
        # %xmm0=sgn(%xmm0)
        movq    %xmm0,%rax            # rax = xmm0
        .if .LBITS64==1
        testq   %rax,%rax             # if double value is zero
        .else
        testl   %eax,%eax             # if float value is zero
        .endif
        jnz     0f                    # then just return that zero and we're done,
                                      # otherwise we must compute X-SGN(X)*INT(ABS(X))
        {disp32} jmp 10f
0:
        pushq   %r15                  # save backup copy of r15     (8)
        movq    %xmm0,%r15            # r15 = X
        pushq   %rdx                  # save backup copy of rdx     (8)
        pushq   %rcx                  # save backup copy of rcx     (8)
        movq    %xmm1,%rdx            # save backup copy
        pushq   %rdx                  # of xmm1                    (16)
        .if .LBITS64==1
        movq    $.LFLOATONE,%rcx      # rcx =  1.0
        movq    $.LFLOATNEGONE,%rdx   # rdx = -1.0
        btq     $63,%rax              # if sign bit is set
        cmovcq  %rdx,%rcx             # rcx = -1.0 else rcx = 1.0
        .else
        movl    $.LFLOATONE,%ecx      # ecx =  1.0f
        movl    $.LFLOATNEGONE,%edx   # edx = -1.0f
        btl     $31,%eax              # if sign bit is set
        cmovcl  %edx,%ecx             # ecx = -1.0 else ecx = 1.0
        .endif
        # at this point SGN(X) is in rcx
        # and X is still in rax AND xmm0
        # rax = ABS(rax) (as a floating point type)
        .if .LBITS64==1
        btrq    $63,%rax              # set bit 63 (sign bit) to zero
        .else
        btrl    $31,%eax              # set bit 31 (sign bit) to zero
        .endif
        movq    %rcx, %xmm1           # %xmm1 = SGN(X)
        movq    %rax, %xmm0           # update xmm0 with ABS(X)
        movabsq $.Lfloor, %rax        # %xmm0=floor(%xmm0), i.e. INT(ABS(X))
        callq   *%rax
        # xmm0 = SGN(X)*INT(ABS(X))
        # %xmm0 *= %xmm1
        movq    %xmm0,%r12            # backup copy of multiplicand in r12
                                      # since a failed multiply will destroy %xmm0
        movq    %xmm1,%r11            # backup copy of multiplier in r11
                                      # since a failed multiply will destroy %xmm1
                                      # %xmm0=%xmm0*%xmm1
        .if .LBITS64==1
        mulsd   %xmm1,%xmm0           #  xmm0 *= xmm1 (double)
        .else
        mulss   %xmm1,%xmm0           #  xmm0 *= xmm1 (float)
        .endif
        subq    $16,%rsp              # allocate storage for temp copy of MXCSR + 12 bytes padding
        stmxcsr (%rsp)                # temp = mxcsr
        testl   $.LFE_INVALID,(%rsp)  # if temp.FE_INVALID
        jnz     2f                    # then goto 2
        testl   $.LFE_UNDERFLOW,(%rsp)  # if temp.FE_UNDERFLOW
        jnz     3f                    # then goto 3
        testl   $.LFE_OVERFLOW,(%rsp) # if temp.FE_OVERFLOW
        jnz     4f                    # then goto 4 (all OK)
        # if answer is +/- Infinity, an overflow exception must be generated
        .if .LBITS64==1
        movq    %xmm0,%r12            # r12 = xmm0 (double)
        btrq    $63,%r12              # r12 = abs(r12) (double) mask out sign
        movabsq $.LPInf,%r11          # r11 = +INF (double)
        cmpq    %r11,%r12             # if abs(xmm0) = +INF (double)
        .else
        movd    %xmm0,%r12d           # r12 = xmm0 (float)
        btrl    $31,%r12d             # r12 = abs(r12) (float) mask out sign
        movl    $.LPInf,%r11d         # r11 = +INF (float)
        cmpl    %r11d,%r12d           # if abs(xmm0) != +INF (float)
        .endif
        je      1f                    # then goto 7 (answer is OK)
        {disp8} jmp 7f
1:
        {disp8} jmp 5f                # else goto 5 (we need an overflow exception)
2:
        # xmm0 value is invalid
        addq    $16,%rsp              # deallocate storage for temp copy of MXCSR + 12 bytes padding
        movq    %xmm0,%r14            # r14 = xmm0
        .if .LBITS64==1
        movabsq $.LQNaN_Indefinite, %r13    # r13 = QNaN Indefinite (double)
        cmpq    %r13,%r14             # if result is a NaN
        .else
        movl    $.LQNaN_Indefinite, %r13d   # r13 = QNaN Indefinite (float)
        cmpl    %r13d,%r14d           # if result is a NaN
        .endif
        movabsq $.LNaN_msg, %rcx      # then we output "NAN GENERATED"
        movabsq $.Luninitialized_msg, %rdx  # otherwise we output
        cmovne  %rdx,%rcx             # "READ OF UNINITIALIZED VARIABLE"
        movabsq $.Lbadxit_msg, %rax
        jmpq    *%rax                 # DOES NOT RETURN
3:
        # underflow, returns 0 and throws underflow exception
        andl    $0xFFC0,(%rsp)        # clear exception in memory copy of MXCSR
        ldmxcsr (%rsp)                # and reload that into real MXCSR
        # substitute a zero value
        .if .LBITS64==1
        xorpd   %xmm0,%xmm0           # xmm0 = 0 (double)
        .else
        xorps   %xmm0,%xmm0           # xmm0 = 0 (float)
        .endif
        movabsq $.Lunderflow_msg,%rsi # rsi = address of underflow_msg
        {disp8} jmp 6f
4:
        # overflow, returns +/- INF and throws OVERFLOW exception
        andl    $0xFFC0,(%rsp)        # clear exception in memory copy of MXCSR
        ldmxcsr (%rsp)                # and reload that into real MXCSR
        .if .LBITS64==1
        movq    %r11,%r15             # r15 = r11    (double)
        shrq    $63,%r15              # r15 >>= 63   (double)
        movq    %r12,%r13             # r13 = r12    (double)
        shrq    $63,%r13              # r13 >>= 63   (double)
        addq    %r13,%r15             # r15 += r13   (double)
                                      # sum is 0,2 (same signs) or 1 (different signs)
                                      # if same sign return +INF else return -INF
        movabsq $.LPInf,%r13          # r13=+INF     (double)
        movabsq $.LNInf,%r14          # r14=-INF     (double)
        andq    $1,%r15               # mask out all but the last bit
        testq   %r15 ,%r15            # if all zeros (same sign)
        cmovzq  %r13 ,%r14            # then r14 = +INF, else r14 = -INF  (double)
        movq    %r14,%xmm0            # xmm0 = r14   (double)
        .else
        movl    %r11d,%r15d           # r15 = r11    (float)
        shrl    $31,%r15d             # r15 >>= 63   (float)
        movl    %r12d,%r13d           # r13 = r12    (float)
        shrl    $31,%r13d             # r13 >>= 63   (float)
        addl    %r13d,%r15d           # r15 += r13   (float)
                                      # 0,2 (same sign) or 1 (different sign)
                                      # if %r15 is one use -INF else use +INF
        movl    $.LPInf,%r13d         # r13d=+INF    (float)
        movl    $.LNInf,%r14d         # r14d=-INF    (float)
        andl    $1,%r15d              # mask out all but the last bit
        testl   %r15d,%r15d           # if all zeros (same sign)
        cmovzl  %r13d,%r14d           # then r14 = +INF, else r14 = -INF  (float)
        movd    %r14d,%xmm0           # xmm0 = r14   (float)
        .endif
        # fall through intentionally
5:
        # emit overflow exception
        movabsq $.Loverflow_msg,%rsi  # rsi = address of overflow_msg
        # fall through intentionally
6:
        # emit some kind of exception
        movabsq $.Lexception_non_fatal,%r15 # call exception_non_fatal
        callq   *%r15
        # fall through intentionally
7:
        addq    $16,%rsp              # deallocate storage for temp copy of MXCSR + 12 bytes padding
        movq    %xmm0, %xmm1 # xmm1 = SGN(X)*INT(ABS(X))
        movq    %r15,%xmm0   # xmm0 = X
        # %xmm0 -= %xmm1
        movq    %xmm0,%r12            # backup copy of subtrahend in r12
                                      # since a failed subtract will destroy %xmm0
        .if .LBITS64==1
        subsd   %xmm1,%xmm0           #  xmm0 -= xmm1 (double)
        .else
        subss   %xmm1,%xmm0           #  xmm0 -= xmm1 (float)
        .endif
        subq    $4,%rsp               # allocate storage for temp copy of MXCSR
        stmxcsr (%rsp)                # temp = mxcsr
        testl   $.LFE_INVALID,(%rsp)  # if !temp.FE_INVALID
        jz      8f                    # then goto 9
        # OK, result was invalid - why?
        addq    $4,%rsp               # deallocate storage for temp copy of MXCSR
        movq    %xmm0,%r14            # r14 = xmm0 (result of subtraction at this point)
        .if .LBITS64==1
        movabsq $.LQNaN_Indefinite, %r13    # r13 = QNaN Indefinite (double)
        cmpq    %r13,%r14             # if result is a NaN
        .else
        movl    $.LQNaN_Indefinite, %r13d   # r13 = QNaN Indefinite (float)
        cmpl    %r13d,%r14d           # if result is a NaN
        .endif
        movabsq $.LNaN_msg, %rcx      # then we output "NAN GENERATED"
        movabsq $.Luninitialized_msg, %rdx  # otherwise we output
        cmovne  %rdx,%rcx             # "READ OF UNINITIALIZED VARIABLE"
        movabsq $.Lbadxit_msg, %rax
        jmpq    *%rax                 # DOES NOT RETURN
8:                                    # was result overflow?
        testl   $.LFE_OVERFLOW,(%rsp) # if !temp.LE_OVERFLOW
        jz      9f                    # then goto 9
        andl    $0xFFC0,(%rsp)        # clear exception in memory copy of MXCSR
        ldmxcsr (%rsp)                # and reload that into real MXCSR
        movabsq $.Loverflow_msg,%rsi  # rsi = address of overflow error message
        movabsq $.Lexception_non_fatal,%r15 # call exception_non_fatal
        callq   *%r15
        # now we need to return +/-INF
        .if .LBITS64==1
        movabsq $.LNInf,%r13          # r13=-INF (double)
        movabsq $.LPInf,%r14          # r14=+INF (double)
        btq     $63,%r12              # if subtrahend was negative
        cmovcq  %r13 ,%r14            # then use -INF else use +INF
        movq    %r14,%xmm0            # xmm0 = r14 (double)
        .else
        movl    $.LNInf,%r13d         # r13d=-INF (float)
        movl    $.LPInf,%r14d         # r14d=+INF (float)
        btl     $31,%r12d             # if subtrahend was negative
        cmovcl  %r13d,%r14d           # then use -INF else use +INF
        movd    %r14d,%xmm0           # xmm0 = r14 (float)
        .endif
        # fall through to exit
9:      addq    $4,%rsp               # deallocate storage for copy of MXCSR
        popq    %rcx                  # restore backup copy
        movq    %rcx, %xmm1           # of xmm1
        popq    %rcx                  # restore rcx
        popq    %rdx                  # restore rdx
        popq    %r15                  # restore r15
10:
        popq    %rax                  # restore rax
        popfq                         # restore flags
        movq    %rbp, %rsp            # free local variables and destroy
        popq    %rbp                  # the stack frame
        retq                          # and finally return
        .size   .LFP, .-.LFP
