#       .equiv .LSPACE, 32
#       .equiv .LNEWLINE, 10
#       .equiv .LSTDOUT, 1
#       .equiv .LSTDERR, 2
#       .equiv .L__NR_write, 1

        # begin PRINT ZONE code

        # Number of ASCII characters per input line
        .equiv .LMAX_INPUT_BYTES, 80
        .if .LWIDE==1
        # Number of ASCII characters per output line
        .equiv .LOUTPUT_WIDTH, 132
        # Number of significant digits
        .equiv .LSIGNIFICANCE_WIDTH, 18
        # Number of exponent digits
        .equiv .LEXRAD_WIDTH, 3
        # Need room for leading sign/space, the E and it's sign/space,
        # and a decimal, and a trailing space
        .equiv .LZONE_WIDTH, .LSIGNIFICANCE_WIDTH+.LEXRAD_WIDTH+5
        .else
        # Number of ASCII characters per output line
        .equiv .LOUTPUT_WIDTH, 80
        # Number of significant digits
        .equiv .LSIGNIFICANCE_WIDTH, 7
        # Number of exponent digits
        .equiv .LEXRAD_WIDTH, 2
        # Need room for leading sign/space, the E and it's sign/space,
        # and a decimal, and a trailing space
        .equiv .LZONE_WIDTH, .LSIGNIFICANCE_WIDTH+.LEXRAD_WIDTH+6
        .endif
        # Maximum zone number, zones go from 1 to .LMAX_ZONE
        .equiv .LMAX_ZONE, .LOUTPUT_WIDTH/.LZONE_WIDTH
        # 8 ASCII spaces
        .equiv .LEIGHT_SPACES, 0x2020202020202020

        .section .data,"aw",@progbits

        .align 8

        .type   .Lobuf_next, @object
.Lobuf_next:
        # Position of next byte of output buffer to use
        .quad   1
        .size   .Lobuf_next, .-.Lobuf_next

        .type   .Lcurzone, @object
.Lcurzone:
        # Current zone to use for output
        .quad   1
        .size   .Lcurzone, .-.Lcurzone

.LOBUF:
        # the actual one-line output buffer
        .asciz  "OUTPUT BUFFER IS UNINITIALIZED"
        .zero   .LOUTPUT_WIDTH-30+2
        .size   .LOBUF, .-.LOBUF

        .type   .Lebuf_next, @object
.Lebuf_next:
        # Position of next byte of error output buffer to use
        .quad   1
        .size   .Lebuf_next, .-.Lebuf_next

        .type   .Lcurezone, @object
.Lcurezone:
        # Current zone to use for error output
        .quad   1
        .size   .Lcurezone, .-.Lcurezone

.LEBUF:
        # the actual one-line error output buffer
        .asciz  "ERROR BUFFER IS UNINITIALIZED"
        .zero   .LOUTPUT_WIDTH-29+2
        .size   .LEBUF, .-.LEBUF

        .section .text,"ax",@progbits
#
# This procedure will initialize the output system for a buffer.  It will
# first initialize the output buffer to all spaces.  Then it will set the
# current output position in the buffer to 1, and finally set the current
# output zone to 1.  Columns and zones are 1-based.
#
# rdi is output buffer variable address (&OBUF)
# rsi is output buffer position variable address (&obuf_next)
# rdx is output zone variable address (&curzone)
#
        .type   .Linitbuffer, @function
.Linitbuffer:
        pushq   %rax                    # save rax register
        pushq   %rbx                    # save rbx register
        pushq   %rcx                    # save rcx register
        pushq   %rdx                    # save output zone variable address
        pushq   %rsi                    # save output buffer position variable address
        cld                             # set forward direction
        xorl    %edx,%edx               # rdx = 0
        movl    $.LOUTPUT_WIDTH, %eax   # how many spaces
        movl    %eax,%edx               # edx = eax
        shrl    $3,%eax                 # eax = .LOUTPUT_WIDTH // 8
        movl    %eax,%ebx               # ebx = eax
        shll    $3,%ebx                 # ebx = ebx * 8
        subl    %ebx,%edx               # edx = edx - ebx, so edx = .LOUTPUT_WIDTH % 8
        pushq   %rax                    # save (.LOUTPUT_WIDTH // 8) on stack
        testl   %edx,%edx               # if rdx == 0 ((.LOUTPUT_WIDTH % 8) == 0)
        jz      0f                      # then goto 0
        # do remaining dx bytes first (if any) (<8 bytes)
        movl    %edx, %ecx              # ecx = remainder (number of bytes left to initialize)
        movb    $.LSPACE, %al           # al = ' '
rep     stosb                           # store (OUTPUT_WIDTH % 8) spaces in buffer at rdi
0:
        # now to ax 8 byte blocks (multiple of 8 bytes (8 bytes * 8bits/byte = 64bits, quadword)
        popq    %rcx                    # rcx = (.LOUTPUT_WIDTH // 8)
        testl   %ecx,%ecx               # if rcx == 0 ((.LINPUT_WIDTH // 8) == 0)
        jz      1f                      # then goto 1
        movabsq $.LEIGHT_SPACES, %rax   # rax = '        '
rep     stosq                           # store (OUTPUT_WIDTH // 8) copies of '        ' in buffer at rdi
1:
        movb    $0, (%rdi)              # and null-terminate that string
        popq    %rsi                    # rsi is output buffer position variable address
        movq    $1, (%rsi)              # initialize that buffer position variable to 1
        popq    %rdx                    # rdx is output zone variable address
        movq    $1, (%rdx)              # initialize that zone variable to 1
        popq    %rcx                    # restore rcx register
        popq    %rbx                    # restore rbx register
        popq    %rax                    # restore rax register
        retq
        .size   .Linitbuffer, .-.Linitbuffer

#
# This procedure will first add a newline to the output buffer contents.
# Then it writes the modified output buffer contents to .LSTDOUT.  Finally,
# the output buffer is re-initialized by calling the initbuffer procedure.
#
# rdi is output buffer variable address (&OBUF)
# rsi is output buffer position variable address (&obuf_next)
# rdx is output zone variable address (&curzone)
#
        .type   .Loutputbuffer, @function
.Loutputbuffer:
        pushq   %rbp                         # create the
        movq    %rsp,%rbp                    # stack frame
        subq    $24,%rsp                     # create three 8-byte local variables
        movq    %rdi,-8(%rbp)                # save output buffer variable address (&OBUF)
        movq    %rsi,-16(%rbp)               # save output buffer position variable address (&obuf_next)
        movq    %rdx,-24(%rbp)               # save output zone variable address (&curzone)
        movb    $.LNEWLINE, .LOUTPUT_WIDTH(%rdi) # add a newline at the end of the line
        movl    $.LOUTPUT_WIDTH+1, %edx      # rdx is number of bytes to output
        movq    %rdi, %rsi                   # rsi is the address of the output buffer
        movl    $.LSTDOUT, %edi              # rdi is .LSTDOUT, the output file
        movq    $.L__NR_write, %rax          # call write system call
        syscall                              # to actually show the output
        movq    -8(%rbp),%rdi                # restore output buffer variable address (&OBUF)
        movq    -16(%rbp),%rsi               # restore output buffer position variable address (&obuf_next)
        movq    -24(%rbp),%rdx               # restore output zone variable address (&curzone)
        movabsq $.Linitbuffer,%rax           # call initbuffer to re-initialize output
        callq   *%rax                        # buffer and reset position and zone
        movq    %rbp,%rsp                    # destroy the stack frame which
        popq    %rbp                         # will destroy the local variables too
        retq
        .size   .Loutputbuffer, .-.Loutputbuffer

#
# This procedure will first terminate the output buffer contents so that
# it is a valid ASCIIZ string.  Then it writes the output buffer contents
# to .LSTDOUT, followed by a newline.  If the buffer is empty, then the
# output is skipped.  Either way, the output buffer is re-initialized
# by calling the initbuf() procedure.
#
# rdi is output buffer variable address (&OBUF)
# rsi is output buffer position variable address (&obuf_next)
# rdx is output zone variable address (&curzone)
#
        .type   .Lflushoutputln, @function
.Lflushoutputln:
        pushq   %rbp                         # create the
        movq    %rsp,%rbp                    # stack frame
        subq    $24,%rsp                     # create three 8-byte local variables
        movq    %rdi,-8(%rbp)                # save buffer variable address (&OBUF)
        movq    %rsi,-16(%rbp)               # save buffer position variable address (&obuf_next)
        movq    %rdx,-24(%rbp)               # save zone variable address (&curzone)
        cmpq    $1,(%rsi)                    # if buffer position is 1 then
        je      0f                           # goto following 0
        movb    $.LNEWLINE, .LOUTPUT_WIDTH(%rdi) # add a newline at the end of the line
        movl    $.LOUTPUT_WIDTH+1, %edx      # rdx is number of bytes to output
        movq    %rdi, %rsi                   # rsi is the address of the output buffer
        movl    $.LSTDOUT, %edi              # rdi is .LSTDOUT, the output file
        movq    $.L__NR_write, %rax          # call write system call
        syscall                              # to actually show the output
0:
        movq    -8(%rbp),%rdi                # restore buffer variable address (&OBUF)
        movq    -16(%rbp),%rsi               # restore buffer position variable address (&obuf_next)
        movq    -24(%rbp),%rdx               # restore zone variable address (&curzone)
        movabsq $.Linitbuffer,%rax           # call initbuffer to re-initialize output
        callq   *%rax                        # buffer and reset position and zone
        movq    %rbp,%rsp                    # destroy the stack frame which
        popq    %rbp                         # will destroy the local variables too
        retq
        .size   .Lflushoutputln, .-.Lflushoutputln

        .section .rodata.str,"aS",@progbits

        .type   .LAPPENDBUFERRMSG1,@object
.LAPPENDBUFERRMSG1:
        .asciz  "NULL POINTER SENT TO APPENDBUF"
        .set    .LAPPENDBUFERRMSG1len, .-.LAPPENDBUFERRMSG1

        .section .text,"ax",@progbits
#
# This procedure will cause the ASCIIZ string specified by the rcx parameter
# to be output to .LSTDOUT through the buffered print system.
# Columns and zones are 1-based.
#
# rdi is output buffer variable address (&OBUF)
# rsi is output buffer position variable address (&obuf_next)
# rdx is output zone variable address (&curzone)
# rcx is input string buffer address
#
# clobbers rax, rcx, rdx, rdi, rsi
#
        .type   .Lappendbuffer, @function
.Lappendbuffer:
        pushq   %rbp                         # create the
        movq    %rsp,%rbp                    # stack frame
        subq    $32,%rsp                     # create 4 8-byte local variables
        movq    %rdi,-8(%rbp)                # save output buffer variable address (&OBUF)
        movq    %rsi,-16(%rbp)               # save output buffer position variable address (&obuf_next)
        movq    %rdx,-24(%rbp)               # save output zone variable address (&curzone)
        movq    %rcx,-32(%rbp)               # save input string variable address
        pushq   %r14                         # save a backup copy of r14 value
        pushq   %r13                         # save a backup copy of r13 value
        pushq   %r12                         # save a backup copy of r12 value
        pushq   %r8                          # save a backup copy of r8 value
        pushq   %rbx                         # save a backup copy of rbx value
        movq    %rcx,%r12                    # r12 = input string variable address
        testq   %rcx,%rcx                    # input string variable address is not NULL then
        jnz     0f                           # goto following 0
        movl    $.LAPPENDBUFERRMSG1len,%edx  # rdx is length of error message about NULL string buffer in bytes
        movl    $.LSTDERR,%edi               # rdi is file number of .LSTDERR
        movabsq $.LAPPENDBUFERRMSG1,%rsi     # rsi is address of that error message
        movq    $.L__NR_write,%rax           # call kernel
        syscall                              # write function
        movabsq $.Lbadxit,%rax               # and then
        jmpq    *%rax                        # goto badexit
0:
        movq    -24(%rbp),%rax               # rax is output zone variable address (&curzone)
        movq    (%rax),%rax                  # rax is value of curzone (newpos)
        decl    %eax                         # rax is (curzone-1)
        movl    $.LZONE_WIDTH, %r8d          # r8 = .LZONE_WIDTH
        mulq    %r8                          # rdx:rax = (curzone - 1)*.LZONE_WIDTH         (rdx will be zero)
        incl    %eax                         # rax = ((curzone - 1)*.LZONE_WIDTH) + 1       (newpos)
        movq    -16(%rbp),%rdx               # rdx is output buffer position variable address (&obuf_next)
        cmpq    %rax,(%rdx)                  # if (obuf_next < newpos)
        jae     1f                           # then
        movq    %rax,(%rdx)                  #   newpos = obuf_next
1:                                           # endif
        movq    -32(%rbp),%rdi               # rdi = input string variable address
        movabsq $.Lmystrlen,%rax             # rax = address of mystrlen
        callq   *%rax                        # call mystrlen
        movq    %rax,%r13                    # r13 = length of input string (slen)
        movq    -16(%rbp),%rax               # rax = output buffer position variable address (&obuf_next)
        movq    (%rax),%rax                  # rax = value of output buffer position variable (obuf_next)
        cmpq    $1,%rax                      # if obuf_next <= 1 then
        jbe     2f                           # jump to following 2
        leaq    -1(%r13,%rax),%rax           # if (obuf_next+slen-1)
        cmpq    $.LOUTPUT_WIDTH,%rax         #    <= .LOUTPUT_WIDTH then
        jbe     2f                           # jump to following 2
        movq    -8(%rbp),%rdi                # rdi = output buffer variable address (&OBUF)
        movq    -16(%rbp),%rsi               # rsi = output buffer position variable address (&obuf_next)
        movq    -24(%rbp),%rdx               # rdx = output zone variable address (&curzone)
        movabsq $.Loutputbuffer,%rax         # call the outputbuffer
        callq   *%rax                        # subroutine
2:
        movq    -8(%rbp),%r14                # r14 = output buffer variable address (&OBUF)
        xorl    %ebx,%ebx                    # rbx holds i, i = 0
        {disp8} jmp 4f                       # goto following 4
3:                                           # beginning of loop body
        incq    %rbx                         # i++
        movq    -16(%rbp),%r8                # r8=output buffer position variable address (&obuf_next)
        leaq    1(%rsi),%rdx                 # rdx=obuf_next+1, then obuf_next=rdx, so
        movq    %rdx,(%r8)                   # obuf_next++
        movb    -1(%r12,%rbx),%cl            # cl=s[i-1]
        movb    %cl,-1(%r14,%rsi)            # OBUF[obuf_next-1]=cl
        cmpq    %r13,%rbx                    # if (i>=slen) then
        jae     4f                           # goto following 4
        cmpq    $.LOUTPUT_WIDTH,%rdx         # if (obuf_next-1<=.LOUTPUT_WIDTH) then
        jbe     4f                           # goto following 4
        movq    -8(%rbp),%rdi                # rdi = output buffer variable address (&OBUF)
        movq    -16(%rbp),%rsi               # rsi = output buffer position variable address (&obuf_next)
        movq    -24(%rbp),%rdx               # rdx = output zone variable address (&curzone)
        movabsq $.Loutputbuffer,%rax         # call the outputbuffer
        callq   *%rax                        # subroutine
4:                                           # end loop body, start loop test
        movq    -16(%rbp),%rsi               # rsi = output buffer position variable address (&obuf_next)
        movq    (%rsi),%rsi                  # rsi = output buffer position
        cmpq    %r13,%rbx                    # if i != slen then
        jne     3b                           # goto previous 3
                                             # end loop test
        leaq    -1(%rsi),%rax                # rax = rsi-1         , rax = (obuf_next-1)
        movl    $.LZONE_WIDTH,%ecx           # ecx = .LZONE_WIDTH
        xorl    %edx,%edx                    # rdx = 0
        divq    %rcx                         # rax = rdx:rax / rcx , (obuf_next-1)/.LZONE_WIDTH
        incl    %eax                         # rax += 1            , rax = ((obuf_next-1)/.LZONE_WIDTH+1)
        movq    -24(%rbp),%r12               # r12 = output zone variable address (&curzone)
        movq    %rax,(%r12)                  # curzone= ((obuf_next-1)/.LZONE_WIDTH+1)
        popq    %rbx                         # restore rbx value
        popq    %r8                          # restore r8 value
        popq    %r12                         # restore r12 value
        popq    %r13                         # restore r13 value
        popq    %r14                         # restore r14 value
        movq    %rbp,%rsp                    # destroy the stack frame which
        popq    %rbp                         # will destroy the local variables too
        retq
        .size   .Lappendbuffer, .-.Lappendbuffer

#
# This procedure will update curzone to the next avaiable zone, wrapping
# back to zone 1 after .LMAX_ZONE.
#
# rdi is output buffer variable address (&OBUF)
# rsi is output buffer position variable address (&obuf_next)
# rdx is output zone variable address (&curzone)
#
        .type   .Lnextzone, @function
.Lnextzone:
        pushq   %rbp                         # create the
        movq    %rsp,%rbp                    # stack frame
        subq    $24,%rsp                     # create three 8-byte local variables
        movq    %rdi,-8(%rbp)                # save output buffer variable address (&OBUF)
        movq    %rsi,-16(%rbp)               # save output buffer position variable address (&obuf_next)
        movq    %rdx,-24(%rbp)               # save output zone variable address (&curzone)
        movq    (%rdx),%rax                  # rax is current zone number (curzone)
        leaq    1(%rax), %rax                # nextzone=curzone+1
        cmpq    $.LMAX_ZONE, %rax            # if nextzone <= .LMAX_ZONE then
        jbe     1f                           # goto following 1
        movabsq $.Loutputbuffer, %rax        # call outputbuffer to output
        call    *%rax                        # the current buffered line
        movl    $1, %eax                     # nextzone=1
1:
        movq    -24(%rbp),%rdx               # rdx=address of output zone variable (&curzone)
        movq    %rax, (%rdx)                 # curzone=nextzone
        movl    $.LZONE_WIDTH, %ecx          # rcx=.LZONE_WIDTH
        mulq    %rcx                         # rdx:rax*=rcx       (rdx will be zero)
        leaq    -.LZONE_WIDTH(%rax), %rax    # rax=rax-.LZONE_WIDTH
        movq    -16(%rbp),%rdx               # rdx is address of output buffer position variable (&obuf_next)
        movq    %rax, (%rdx)                 # obuf_next=(curzone-1)*.LZONE_WIDTH
2:
        movq    %rbp,%rsp                    # destroy the stack frame which
        popq    %rbp                         # will destroy the local variables too
        retq
        .size   .Lnextzone, .-.Lnextzone
