#
# function to compute floor() of the value in xmm0
#
        .section .rodata.str,"aS",@progbits

        .type .Lintfail_msg, @object
.Lintfail_msg:
        .asciz "UNABLE TO COMPUTE INT("
        .size .Lintfail_msg, .-.Lintfail_msg

        .type .Lintfunction_msg, @object
.Lintfunction_msg:
        .asciz ") FUNCTION"
        .size .Lintfunction_msg, .-.Lintfunction_msg

        .section .text,"ax",@progbits

        # incoming data is float in xmm0
        # function will calculate floor() of that value and
        # replace the original value in xmm0 with the new value
        .type   .Lfloor, @function
        .balign 16, 0x90
.Lfloor:
        pushq   %rbp                    # create stack frame
        movq    %rsp, %rbp

        .if .LSSE4_1==1
#------------------ with SSE 4.1 --------------------
#
# This means there is a native roundsd instruction available, so
# use it with rounding mode set to round towards -INF
#
        subq    $8, %rsp                # make room for two 4-byte fields
                                        # -4(%rsp) is original mxcsr to restore
                                        # 0(%rsp) is scratch mxcsr
        stmxcsr -4(%rsp)                # save mxcsr for later
        # $1 is round down toward -INF
        .if .LBITS64==1
        roundsd $1, %xmm0, %xmm0        # xmm0 = floor(xmm0)
        .else
        roundss $1, %xmm0, %xmm0        # xmm0 = floorf(xmm0)
        .endif
        stmxcsr (%rsp)                  # get current mxcsr
        testl   $.LFE_INVALID,(%rsp)    # and if the FE_INVALID exception was not set
        jz      0f                      # then goto 0 and continue normally
        addq    $8, %rsp                # deallocate two 4-byte fields
        {disp8} jmp     3f              # otherwise goto 3 and have a fatal error
0:
        ldmxcsr -4(%rsp)                # restore original mxcsr
        addq    $8, %rsp                # deallocate two 4-byte fields
#-------------------------------------------------------
        .else
#------------------ without SSE 4.1 --------------------
#
# This means there is no roundsd instruction available, so
# convert double->integer->double with rounding mode set to
# round towards -INF instead
#
        pushq   %r13                    # save r13
        subq    $8, %rsp                # make room for 2 4-byte fields
                                        # -4(%rsp) is original mxcsr to restore
                                        #  0(%rsp) is temporary mxcsr to modify
        stmxcsr -4(%rsp)                # save mxcsr for later
        stmxcsr (%rsp)                  # save copy of mxcsr
        movl    (%rsp), %r13d           # put current rounding mode into ebx
        andl    $0xffff9fff, %r13d      # clear rouding bits 13/14
        orl     $0x00002000, %r13d      # set rounding down
        movl    %r13d, (%rsp)           # save new rounding mode into temporary mxcsr
        ldmxcsr (%rsp)                  # load new rounding mode from temporary mxcsr
        .if .LBITS64==1
        cvtsd2si %xmm0, %r13            # convert double to 64bit integer
        .else
        cvtss2si %xmm0, %r13            # convert float to 64bit integer
        .endif
        stmxcsr (%rsp)                  # save current mxcsr into temporary mxcsr
        testl   $.LFE_INVALID,(%rsp)    # and if the FE_INVALID exception was not set
        jz      1f                      # then goto 1
        movq    $.LINT_Indefinite,%rax  # Did we get indefinite integer result?
        cmpq    %r13, %rax
        je      2f                      # Yes we did, so just return and leave xmm0 unmodified
        {disp8} jmp     3f              # No we did not, goto fatal error routine
1:
        .if .LBITS64==1
        cvtsi2sd %r13, %xmm0            # convert 64bit integer back to double
        .else
        cvtsi2ss %r13, %xmm0            # convert 64bit integer back to float
        .endif
2:
        ldmxcsr -4(%rsp)                # restore original mxcsr
        addq    $8, %rsp                # deallocate 2 4-byte fields
        popq    %r13                    # restore r13
#-------------------------------------------------------
        .endif
        movq    %rbp,%rsp               # remove stack frame
        popq    %rbp
        retq
3:
        # some other problem
        .if .LBITS64==1
        ldmxcsr (%rsp)                  # restore original mxcsr
        addq    $4, %rsp                # deallocate 4-byte field
        .else
        ldmxcsr -4(%rsp)                # restore original mxcsr
        addq    $8, %rsp                # deallocate 2 4-byte fields
        .endif
        popq    %r13                    # restore r13
        # Output any pending line
        movabsq $.LOBUF,%rdi            # output buffer variable address
        movabsq $.Lobuf_next,%rsi       # output bufpos variable address
        movabsq $.Lcurzone,%rdx         # output zonepos variable address
        movabsq $.Lflushoutputln,%rax
        callq   *%rax
        # Output error message
        movabsq $.Lintfail_msg,%rcx     # error message string address
        movabsq $.LEBUF,%rdi            # error output buffer variable address
        movabsq $.Lebuf_next,%rsi       # error output bufpos variable address
        movabsq $.Lcurezone,%rdx        # error output zonepos variable address
        movabsq $.Lappendbuffer,%rax
        callq   *%rax
        movabsq $.LEBUF,%rdi            # error output buffer variable address
        movabsq $.Lebuf_next,%rsi       # error output bufpos variable address
        movabsq $.Lcurezone,%rdx        # error output zonepos variable address
        movabsq $.Lappendfloat,%rax
        callq   *%rax
        movabsq $.Lintfunction_msg,%rcx # error message string addresss
        movabsq $.Lbadxit_msg, %rax
        callq   *%rax                   # DOES NOT RETURN
        .size .Lfloor, .-.Lfloor
