#
# Copyright (C) 2017,2018,2019,2020,2021  John Gatewood Ham
#
# This file is part of ecma55.
#
# ecma55 is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# ecma55 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ecma55.  If not, see <http://www.gnu.org/licenses/>.
#
# NOTE: This Makefile assumes GNU make >= 4.0 is being used
#
# location = $(word $(words $(MAKEFILE_LIST)),$(MAKEFILE_LIST))
location = $(lastword $(MAKEFILE_LIST))
ME := $(location)
.SUFFIXES:
.DEFAULT_GOAL:	all
export SHELL=/bin/bash
MAKEFLAGS+=--no-print-directory
DESTDIR=
PREFIX=/usr/local
BINDIR=$(PREFIX)/bin
MANDIR=$(PREFIX)/man/man1
VALGRIND=valgrind
VALGRIND_OPTIONS=--leak-check=full --show-leak-kinds=all --track-origins=yes
GROFF=groff
GZIP=gzip
INSTALL=install
INSTALLDIR=$(INSTALL) -d
INSTALLDATA=$(INSTALL) -m 644
INSTALLEXE=$(INSTALL) -m 755
MANPAGES=ecma55.1.gz BASICC.1.gz BASICCS.1.gz BASICCW.1.gz
CC=tcc
AS=as
ASFLAGS=--64 -n --divide --size-check=error
include ./AMD64/ASM_FILELIST
# STD
# OPT
# CPPFLAGS
# CFLAGS2
# CFLAGS
STD?=-std=c11
# DEBUG : debugging using bounds checking
# DEBUG2: debugging for valgrind or gdb
# DEBUG3: debugging with MJOLNIR heap interceptor
ifeq ($(COMPILE_MODE),DEBUG)
  $(info Notice: $(CC) does not support address sanizer)
  DEBUG=-b
  LDFLAGS=-b -bt50
  OPT=-O0
else ifeq ($(COMPILE_MODE),DEBUG2)
  DEBUG=-g
  LDFLAGS=-g -bt50
  OPT=-O0
else ifeq ($(COMPILE_MODE),DEBUG3)
  DEBUG=-g
  override CPPFLAGS+=-DMJOLNIR
  LDFLAGS=-g -bt50
  OPT=-O0
else
  OPT?=-O3
  LDFLAGS=-bt50
endif
CFLAGS2=$(STD) $(OPT) $(DEBUG)
CFLAGS3=-fno-pie -fno-pic -fno-PIC -fno-PIE
override CFLAGS+=$(CFLAGS2) -m64 -Wall -Wunsupported -Wimplicit-function-declaration -Wwrite-strings -fno-common -bt50
LDFLAGS += -Wl,-z,defs -Wl,-z,relro  -Wl,-z,now -Wl,-z,noexecstack
$(info MAKE            = '$(strip $(MAKE))')
$(info MAKEFLAGS       = '$(strip $(MAKEFLAGS))')
$(info ME              = '$(strip $(ME))')
$(info STD             = '$(strip $(STD))')
$(info OPT             = '$(strip $(OPT))')
$(info DEBUG           = '$(strip $(DEBUG))')
$(info AS              = '$(strip $(AS))')
$(info ASFLAGS         = '$(strip $(ASFLAGS))')
$(info CC              = '$(strip $(CC))')
$(info CPPFLAGS        = '$(strip $(CPPFLAGS))')
$(info CFLAGS2         = '$(strip $(CFLAGS2))')
$(info CFLAGS          = '$(strip $(CFLAGS))')
$(info LDFLAGS         = '$(strip $(LDFLAGS))')
$(info COMPILE_MODE    = '$(strip $(COMPILE_MODE))')

.PHONY:	all
all:	ecma55

.PHONY:	install
install:	all $(MANPAGES)
	$(INSTALLDIR) $(DESTDIR)$(BINDIR) $(DESTDIR)$(MANDIR)
	$(INSTALLEXE) ecma55 BASICC BASICCS BASICCW $(DESTDIR)$(BINDIR)/
	$(INSTALLDATA) $(MANPAGES) $(DESTDIR)$(MANDIR)/

.PHONY:	uninstall
uninstall:
	for page in $(MANPAGES); do $(RM) -f $(DESTDIR)$(MANDIR)/$${page}; done
	for binary in ecma55 BASICC BASICCS BASICCW ;\
	  do $(RM) -f $(DESTDIR)$(BINDIR)/$${binary} ;\
	done
	# it's up to you to remove empty directories

parseinput:	parseinput.o
	# tcc cannot link it either....
	gcc -Wl,-z,defs -Wl,-z,relro -Wl,-z,now -no-pie -o $@ $<

parseinput.o:	parseinput.s
	$(AS) $(ASFLAGS) -o $@ $<

parseinput.s:	parseinput.c
	# tcc cannot generate assembly....
	gcc -Os -m64 -mtune=generic -march=x86-64 -mcmodel=large $(CPPFLAGS) -mfpmath=sse -mssse3 \
	    -mno-sse4 -fno-reciprocal-math -fno-associative-math -fno-unwind-tables \
	    -fno-asynchronous-unwind-tables -fno-common -fno-ident \
	    -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -fno-inline -fno-builtin \
	    -fno-optimize-sibling-calls -fno-zero-initialized-in-bss \
	    -fno-stack-protector -mno-red-zone \
	    -fno-jump-tables -fno-delayed-branch -fno-shrink-wrap \
	    -fno-combine-stack-adjustments -fno-guess-branch-probability \
	    -fmerge-constants -fno-delete-null-pointer-checks -fno-schedule-insns \
	    -fno-reorder-blocks-and-partition -fno-reorder-blocks \
	    -fno-align-labels -fno-align-jumps -fno-align-functions \
	    -fno-reorder-functions \
	    -fno-toplevel-reorder -mcld -fno-schedule-fusion -fno-schedule-insns2 \
	    $(CFLAGS3) -DUNIT_TEST -S $< -o $@

peephole:	peephole.c peephole.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST -c $< -o $(<:.c=.o)
	$(CC) $(LDFLAGS) $(filter %.o,$(^:.c=.o)) -o $@

peephole.o:	peephole.c peephole.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

ecma55:	main.o scanner3.o globals.o parser2.o symbol_table.o codegen.o \
	dtoa5_normal.o g_fmt_BASIC_normal.o tree.o \
	optimizer.o ast.o asmgen.o semantic_checks.o error_messages.o \
	dag.o raw_registers.o peephole.o load_textdata.o textdata.o
	$(CC) $(LDFLAGS) $^ -o $@ -lm $(LIBS)

main.o:	main.c globals.h error_messages.h scanner3.h parser2.h symbol_table.h \
	codegen.h peephole.h load_textdata.h textdata.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

symbol_table:	symbol_table.c symbol_table.h globals.h g_fmt_BASIC_normal.h \
        codegen.h load_textdata.h textdata.h load_textdata.o textdata.o \
	globals.o error_messages.o g_fmt_BASIC_normal.o dtoa5_normal.o codegen.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -ldl

symbol_table.o:	symbol_table.c globals.h error_messages.h symbol_table.h \
	g_fmt_BASIC_normal.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

globals.o:	globals.c globals.h error_messages.h raw_registers.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

globals:	globals.c error_messages.o globals.h error_messages.h
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@

error_messages.o:	error_messages.c error_messages.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

codegen.o:	codegen.c globals.h error_messages.h codegen.h \
	load_textdata.h textdata.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

parser2:	parser2.c parser2.h globals.h scanner3.h dtoa5_normal.h \
	g_fmt_BASIC_normal.h tree.h ast.h asmgen.h semantic_checks.h dag.h \
	load_textdata.h textdata.h load_textdata.o textdata.o codegen.o \
	globals.o error_messages.o scanner3.o g_fmt_BASIC_normal.o \
	dtoa5_normal.o tree.o ast.o dag.o semantic_checks.o symbol_table.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm -ldl

parser2.o:	parser2.c parser2.h globals.h error_messages.h scanner3.h \
	dtoa5_normal.h g_fmt_BASIC_normal.h tree.h ast.h asmgen.h \
	semantic_checks.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

asmgen:	asmgen.c asmgen.h globals.h error_messages.h codegen.h \
	symbol_table.h raw_registers.h parser2.h tree.h scanner3.h optimizer.h \
	globals.o codegen.o error_messages.o symbol_table.o raw_registers.o \
	scanner3.o dag.o tree.o optimizer.o g_fmt_BASIC_normal.o dtoa5_normal.o \
	load_textdata.o textdata.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm $(LIBS)

asmgen.o:	asmgen.c asmgen.h globals.h error_messages.h codegen.h \
	symbol_table.h raw_registers.h parser2.h tree.h scanner3.h optimizer.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

semantic_checks.o:	semantic_checks.c globals.h error_messages.h tree.h \
	scanner3.h symbol_table.h semantic_checks.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

semantic_checks:	semantic_checks.c semantic_checks.h tree.h globals.h \
	error_messages.h symbol_table.h scanner3.h g_fmt_BASIC_normal.h dag.h \
        ast.h load_textdata.h textdata.h load_textdata.o textdata.o codegen.o \
	dtoa5_normal.h globals.o error_messages.o symbol_table.o tree.o \
	scanner3.o g_fmt_BASIC_normal.o dtoa5_normal.o dag.o ast.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm $(LIBS)

scanner3:	scanner3.c scanner3.h globals.h globals.o error_messages.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm

scanner3.o:	scanner3.c scanner3.h globals.h error_messages.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

tree:	tree.c tree.h globals.h scanner3.h dtoa5_normal.h error_messages.h \
	globals.o error_messages.o scanner3.o dtoa5_normal.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm

tree.o:	tree.c tree.h globals.h scanner3.h dtoa5_normal.h error_messages.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

ast:	ast.c ast.h tree.h scanner3.h globals.h error_messages.h \
	tree.o scanner3.o globals.o error_messages.o dtoa5_normal.o \
	g_fmt_BASIC_normal.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm

ast.o:	ast.c ast.h tree.h scanner3.h globals.h error_messages.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

dag.o:	dag.c dag.h raw_registers.h globals.h error_messages.h symbol_table.h \
	tree.h scanner3.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

dag:	dag.c globals.o error_messages.o scanner3.o tree.o dtoa5_normal.o g_fmt_BASIC_normal.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm

raw_registers.o:	raw_registers.c raw_registers.h globals.h error_messages.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

raw_registers:	raw_registers.c error_messages.o globals.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm

optimizer:	optimizer.c optimizer.h scanner3.o globals.o symbol_table.o \
	codegen.o dtoa5_normal.o g_fmt_BASIC_normal.o tree.o ast.o asmgen.o \
	semantic_checks.o error_messages.o dag.o raw_registers.o \
	load_textdata.o textdata.o
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ -lm $(LIBS)

optimizer.o:	optimizer.c parser2.h tree.h \
	scanner3.h globals.h error_messages.h g_fmt_BASIC_normal.h \
	optimizer.h symbol_table.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

dumpregs.o:	dumpregs.s
	$(AS) $(ASFLAGS) -o $@ $<

g_fmt_BASIC_normal.o:	g_fmt_BASIC_normal.c g_fmt_BASIC_normal.h dtoa5_normal.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -DIGNORE_ZERO_SIGN -c $< -o $@

dtoa5_normal.o:	dtoa5_normal.c dtoa5_normal.h
	$(CC) $(CFLAGS) -DIEEE_8087 -DLong=int -DPRIVATE_MEM=16384 $(CPPFLAGS) -c $< -o $@

ecma55.1.ps:	ecma55.1
	$(GROFF) -Tps -man $< >$@

zonermore:	zonermore.o
	$(CC) $(LDFLAGS) $< -o $@

zonermore.o:	zonermore.c
	$(CC) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST -c $< -o $@

robert1:	robert1.o
	$(CC) $(LDFLAGS) $< -o $@

robert1.o:	robert1.c
	$(CC) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST -c $< -o $@

load_textdata:	load_textdata.c textdata.o globals.o error_messages.o textdata.h load_textdata.h globals.h error_messages.h
	$(CC) $(LDFLAGS) $(CFLAGS) $(CPPFLAGS) -DUNIT_TEST $< $(filter-out $@.o,$(filter %.o,$(^:.c=.o))) -o $@ $(LIBS)

load_textdata.o:	load_textdata.c textdata.h globals.h load_textdata.h error_messages.h
	$(CC) $(CFLAGS) $(CPPFLAGS) -c $< -o $@

textdata.o:	textdata.s
	$(AS) $(ASFLAGS) -o $@ $<

textdata.s:	textdata.s.in $(addprefix AMD64/,$(ASSEMBLY_TEXT_FILES)) COPYING \
	BOOST_LICENSE-1.0.TXT LUCENT_LICENSE.TXT CC0-1.0-Universal \
	AMD64/ASM_FILELIST
	sed -e "s/FILENAMES/$(ASSEMBLY_TEXT_FILES:.s=,)/g" < $< > $@

textdata.h:	$(addprefix AMD64/,$(ASSEMBLY_TEXT_FILES)) AMD64/ASM_FILELIST
	printf "#ifndef TEXTDATA_H\n" >$@
	printf "#define TEXTDATA_H\n" >>$@
	printf "#define COPYING                        0\n" >>$@
	printf "#define BOOST_LICENSE                  1\n" >>$@
	printf "#define LUCENT_LICENSE                 2\n" >>$@
	printf "#define CC0_LICENSE                    3\n" >>$@
	ndex=4 ;\
	for fname in $(ASSEMBLY_TEXT_FILES:.s=_asmcode); do \
	printf "#define %-30s %u\n" "$${fname}" $${ndex} ;\
	ndex=`expr $$ndex + 1` ;\
	done >>$@
	printf "#endif\n" >>$@

.PHONY:	check_without_valgrind
check_without_valgrind:	check_unit_tests_bounds
	$(info MAKE            = '$(strip $(MAKE))')
	$(info MAKEFLAGS       = '$(strip $(MAKEFLAGS))')
	$(info ME              = '$(strip $(ME))')
	$(info STD             = '$(strip $(STD))')
	$(info OPT             = '$(strip $(OPT))')
	$(info DEBUG           = '$(strip $(DEBUG))')
	$(info AS              = '$(strip $(AS))')
	$(info ASFLAGS         = '$(strip $(ASFLAGS))')
	$(info CC              = '$(strip $(CC))')
	$(info CPPFLAGS        = '$(strip $(CPPFLAGS))')
	$(info CFLAGS2         = '$(strip $(CFLAGS2))')
	$(info CFLAGS          = '$(strip $(CFLAGS))')
	$(info LDFLAGS         = '$(strip $(LDFLAGS))')
	$(info COMPILE_MODE    = '$(strip $(COMPILE_MODE))')
	$(info without Valgrind, 64 bit)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) distclean && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) ecma55 COMPILE_MODE=$(COMPILE_MODE) && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) clean ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=1 ECMA55FLAGS='' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=1 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=2 ECMA55FLAGS='-O1' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=2 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=3 ECMA55FLAGS='-O2' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=3 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=4 ECMA55FLAGS='-X -O3' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=4 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=45 ECMA55FLAGS='-w' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=45 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=5 ECMA55FLAGS='' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=5 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=6 ECMA55FLAGS='-O1' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=6 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=7 ECMA55FLAGS='-O2' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=7 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=8 ECMA55FLAGS='-X -O3' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=8 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=41 ECMA55FLAGS='-w' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=41 ;\
	if grep ^flags /proc/cpuinfo | uniq | grep -qw sse4_1; then \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=9 ECMA55FLAGS='-4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=9 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=10 ECMA55FLAGS='-O1 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=10 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=11 ECMA55FLAGS='-O2 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=11 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=12 ECMA55FLAGS='-X -O3 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=12 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=46 ECMA55FLAGS='-4 -w' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=46 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=13 ECMA55FLAGS='-4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=13 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=14 ECMA55FLAGS='-O1 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=14 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=15 ECMA55FLAGS='-O2 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=15 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=16 ECMA55FLAGS='-X -O3 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=16 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=42 ECMA55FLAGS='-4 -w' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=42 ;\
	fi ;\
	echo without Valgrind, 32 bit ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=25 ECMA55FLAGS='-s' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=25 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=26 ECMA55FLAGS='-s -O1' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=26 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=27 ECMA55FLAGS='-s -O2' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=27 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=28 ECMA55FLAGS='-s -X -O3' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=28 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=150 ECMA55FLAGS='-s -w' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=150 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=29 ECMA55FLAGS='-s' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=29 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=30 ECMA55FLAGS='-s -O1' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=30 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=31 ECMA55FLAGS='-s -O2' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=31 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=32 ECMA55FLAGS='-s -X -O3' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=32 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=43 ECMA55FLAGS='-s -w' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=43 ;\
	if grep ^flags /proc/cpuinfo | uniq | grep -qw sse4_1; then \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=33 ECMA55FLAGS='-s -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=33 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=34 ECMA55FLAGS='-s -O1 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=34 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=35 ECMA55FLAGS='-s -O2 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=35 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=36 ECMA55FLAGS='-s -X -O3 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=36 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=151 ECMA55FLAGS='-s -4 -w' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=151 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=37 ECMA55FLAGS='-s -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=37 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=38 ECMA55FLAGS='-s -O1 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=38 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=39 ECMA55FLAGS='-s -O2 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=39 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=40 ECMA55FLAGS='-s -X -O3 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=40 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=44 ECMA55FLAGS='-s -4 -w' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=44 ;\
	fi

.PHONY:	check_with_bounds
check_with_bounds:	check_with_valgrind
	$(info MAKE            = '$(strip $(MAKE))')
	$(info MAKEFLAGS       = '$(strip $(MAKEFLAGS))')
	$(info ME              = '$(strip $(ME))')
	$(info STD             = '$(strip $(STD))')
	$(info OPT             = '$(strip $(OPT))')
	$(info DEBUG           = '$(strip $(DEBUG))')
	$(info AS              = '$(strip $(AS))')
	$(info ASFLAGS         = '$(strip $(ASFLAGS))')
	$(info CC              = '$(strip $(CC))')
	$(info CPPFLAGS        = '$(strip $(CPPFLAGS))')
	$(info CFLAGS2         = '$(strip $(CFLAGS2))')
	$(info CFLAGS          = '$(strip $(CFLAGS))')
	$(info LDFLAGS         = '$(strip $(LDFLAGS))')
	$(info COMPILE_MODE    = 'DEBUG')
	$(info without Valgrind, 64 bit)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) distclean && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) ecma55 COMPILE_MODE=DEBUG && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) clean ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=1 ECMA55FLAGS='' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=1 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=2 ECMA55FLAGS='-O1' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=2 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=3 ECMA55FLAGS='-O2' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=3 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=4 ECMA55FLAGS='-X -O3' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=4 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=300 ECMA55FLAGS='-w' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=300 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=5 ECMA55FLAGS='' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=5 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=6 ECMA55FLAGS='-O1' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=6 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=7 ECMA55FLAGS='-O2' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=7 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=8 ECMA55FLAGS='-X -O3' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=8 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=241 ECMA55FLAGS='-w' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=241 ;\
	if grep ^flags /proc/cpuinfo | uniq | grep -qw sse4_1; then \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=9 ECMA55FLAGS='-4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=9 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=10 ECMA55FLAGS='-O1 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=10 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=11 ECMA55FLAGS='-O2 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=11 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=12 ECMA55FLAGS='-X -O3 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=12 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=301 ECMA55FLAGS='-4 -w' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=301 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=13 ECMA55FLAGS='-4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=13 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=14 ECMA55FLAGS='-O1 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=14 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=15 ECMA55FLAGS='-O2 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=15 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=16 ECMA55FLAGS='-X -O3 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=16 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=242 ECMA55FLAGS='-4 -w' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=242 ;\
	fi ;\
	echo without Valgrind, 32 bit ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=25 ECMA55FLAGS='-s' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=25 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=26 ECMA55FLAGS='-s -O1' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=26 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=27 ECMA55FLAGS='-s -O2' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=27 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=28 ECMA55FLAGS='-s -X -O3' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=28 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=302 ECMA55FLAGS='-s -w' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=302 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=29 ECMA55FLAGS='-s' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=29 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=30 ECMA55FLAGS='-s -O1' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=30 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=31 ECMA55FLAGS='-s -O2' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=31 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=32 ECMA55FLAGS='-s -X -O3' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=32 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=47 ECMA55FLAGS='-s -w' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=47 ;\
	if grep ^flags /proc/cpuinfo | uniq | grep -qw sse4_1; then \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=33 ECMA55FLAGS='-s -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=33 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=34 ECMA55FLAGS='-s -O1 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=34 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=35 ECMA55FLAGS='-s -O2 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=35 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=36 ECMA55FLAGS='-s -X -O3 -4' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=36 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=303 ECMA55FLAGS='-s -4 -w' && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=303 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=37 ECMA55FLAGS='-s -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=37 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=38 ECMA55FLAGS='-s -O1 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=38 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=39 ECMA55FLAGS='-s -O2 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=39 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=40 ECMA55FLAGS='-s -X -O3 -4' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=40 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=48 ECMA55FLAGS='-s -4 -w' && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=48 ;\
	fi

.PHONY:	check_with_valgrind
check_with_valgrind:	check_without_valgrind
	$(info MAKE            = '$(strip $(MAKE))')
	$(info MAKEFLAGS       = '$(strip $(MAKEFLAGS))')
	$(info ME              = '$(strip $(ME))')
	$(info STD             = '$(strip $(STD))')
	$(info OPT             = '$(strip $(OPT))')
	$(info DEBUG           = '$(strip $(DEBUG))')
	$(info AS              = '$(strip $(AS))')
	$(info ASFLAGS         = '$(strip $(ASFLAGS))')
	$(info CC              = '$(strip $(CC))')
	$(info CPPFLAGS        = '$(strip $(CPPFLAGS))')
	$(info CFLAGS2         = '$(strip $(CFLAGS2))')
	$(info CFLAGS          = '$(strip $(CFLAGS))')
	$(info LDFLAGS         = '$(strip $(LDFLAGS))')
	$(info COMPILE_MODE    = 'DEBUG2')
	$(info with Valgrind, 64 bit)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) distclean && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) ecma55 COMPILE_MODE=DEBUG2 && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) clean ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=101 ECMA55FLAGS='' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=101 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=102 ECMA55FLAGS='-O1' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=102 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=103 ECMA55FLAGS='-O2' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=103 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=104 ECMA55FLAGS='-X -O3' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=104 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=304 ECMA55FLAGS='-w' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=304 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=105 ECMA55FLAGS='' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=105 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=106 ECMA55FLAGS='-O1' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=106 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=107 ECMA55FLAGS='-O2' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=107 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=108 ECMA55FLAGS='-X -O3' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=108 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=140 ECMA55FLAGS='-w' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=140 ;\
	if grep ^flags /proc/cpuinfo | uniq | grep -qw sse4_1; then \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=109 ECMA55FLAGS='-4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=109 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=110 ECMA55FLAGS='-O1 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=110 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=111 ECMA55FLAGS='-O2 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=111 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=112 ECMA55FLAGS='-X -O3 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=112 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=305 ECMA55FLAGS='-4 -w' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=305 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=113 ECMA55FLAGS='-4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=113 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=114 ECMA55FLAGS='-O1 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=114 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=115 ECMA55FLAGS='-O2 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=115 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=116 ECMA55FLAGS='-X -O3 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=116 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=141 ECMA55FLAGS='-4 -w' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=141 ;\
	fi ;\
	echo with Valgrind, 32 bit ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=125 ECMA55FLAGS='-s' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=125 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=126 ECMA55FLAGS='-s -O1' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=126 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=127 ECMA55FLAGS='-s -O2' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=127 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=128 ECMA55FLAGS='-s -X -O3' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=128 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=306 ECMA55FLAGS='-s -w' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=306 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=129 ECMA55FLAGS='-s' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=129 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=130 ECMA55FLAGS='-s -O1' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=130 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=131 ECMA55FLAGS='-s -O2' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=131 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=132 ECMA55FLAGS='-s -X -O3' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=132 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=142 ECMA55FLAGS='-s -w' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=142 ;\
	if grep ^flags /proc/cpuinfo | uniq | grep -qw sse4_1; then \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=133 ECMA55FLAGS='-s -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=133 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=134 ECMA55FLAGS='-s -O1 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=134 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=135 ECMA55FLAGS='-s -O2 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=135 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=136 ECMA55FLAGS='-s -X -O3 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=136 ;\
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=307 ECMA55FLAGS='-s -4 -w' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=307 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=137 ECMA55FLAGS='-s -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=137 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=138 ECMA55FLAGS='-s -O1 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=138 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=139 ECMA55FLAGS='-s -O2 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=139 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=140 ECMA55FLAGS='-s -X -O3 -4' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=140 ;\
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) all DIRNO=144 ECMA55FLAGS='-s -4 -w' VALGRIND=valgrind && \
	$(MAKE) -fMakefile.runtests2 $(filter-out -j%,$(MAKEFLAGS)) distclean DIRNO=144 ;\
	fi

.PHONY:	check_unit_tests
check_unit_tests:
	$(info MAKE            = '$(strip $(MAKE))')
	$(info MAKEFLAGS       = '$(strip $(MAKEFLAGS))')
	$(info ME              = '$(strip $(ME))')
	$(info STD             = '$(strip $(STD))')
	$(info OPT             = '$(strip $(OPT))')
	$(info DEBUG           = '$(strip $(DEBUG))')
	$(info AS              = '$(strip $(AS))')
	$(info ASFLAGS         = '$(strip $(ASFLAGS))')
	$(info CC              = '$(strip $(CC))')
	$(info CPPFLAGS        = '$(strip $(CPPFLAGS))')
	$(info CFLAGS2         = '$(strip $(CFLAGS2))')
	$(info CFLAGS          = '$(strip $(CFLAGS))')
	$(info LDFLAGS         = '$(strip $(LDFLAGS))')
	$(info COMPILE_MODE    = '$(strip $(COMPILE_MODE))')
	$(info unit tests without Valgrind)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) distclean && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) optimizer parser2 ast dag tree scanner3 symbol_table parseinput zonermore robert1 raw_registers asmgen semantic_checks COMPILE_MODE=$(COMPILE_MODE) && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) clean
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) phase4_tests >phase4_tests.out 2>&1 || \
	exit 1 && (sort phase4_tests.out; rm phase4_tests.out)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) asmgen_tests >asmgen_tests.out 2>&1 || \
	exit 1 && (sort asmgen_tests.out; rm asmgen_tests.out)

.PHONY:	check_unit_tests_valgrind
check_unit_tests_valgrind:	check_unit_tests
	$(info MAKE            = '$(strip $(MAKE))')
	$(info MAKEFLAGS       = '$(strip $(MAKEFLAGS))')
	$(info ME              = '$(strip $(ME))')
	$(info STD             = '$(strip $(STD))')
	$(info OPT             = '$(strip $(OPT))')
	$(info DEBUG           = '$(strip $(DEBUG))')
	$(info AS              = '$(strip $(AS))')
	$(info ASFLAGS         = '$(strip $(ASFLAGS))')
	$(info CC              = '$(strip $(CC))')
	$(info CPPFLAGS        = '$(strip $(CPPFLAGS))')
	$(info CFLAGS2         = '$(strip $(CFLAGS2))')
	$(info CFLAGS          = '$(strip $(CFLAGS))')
	$(info LDFLAGS         = '$(strip $(LDFLAGS))')
	$(info COMPILE_MODE    = 'DEBUG2')
	$(info unit tests with Valgrind)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) distclean && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) optimizer parser2 ast dag tree scanner3 symbol_table parseinput zonermore robert1 raw_registers asmgen semantic_checks COMPILE_MODE=DEBUG2 && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) clean
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) phase4_leak_tests VALGRIND="$(VALGRIND)" VALGRIND_OPTIONS="$(VALGRIND_OPTIONS)" >phase4_leak_tests.out 2>&1 || \
	exit 1 && (sort phase4_leak_tests.out; rm phase4_leak_tests.out)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) asmgen_leak_tests VALGRIND="$(VALGRIND)" VALGRIND_OPTIONS="$(VALGRIND_OPTIONS)" >asmgen_leak_tests.out 2>&1 || \
	exit 1 && (sort asmgen_leak_tests.out; rm asmgen_leak_tests.out)

.PHONY:	check_unit_tests_bounds
check_unit_tests_bounds:	check_unit_tests_valgrind
	$(info MAKE            = '$(strip $(MAKE))')
	$(info MAKEFLAGS       = '$(strip $(MAKEFLAGS))')
	$(info ME              = '$(strip $(ME))')
	$(info STD             = '$(strip $(STD))')
	$(info OPT             = '$(strip $(OPT))')
	$(info DEBUG           = '$(strip $(DEBUG))')
	$(info AS              = '$(strip $(AS))')
	$(info ASFLAGS         = '$(strip $(ASFLAGS))')
	$(info CC              = '$(strip $(CC))')
	$(info CPPFLAGS        = '$(strip $(CPPFLAGS))')
	$(info CFLAGS2         = '$(strip $(CFLAGS2))')
	$(info CFLAGS          = '$(strip $(CFLAGS))')
	$(info LDFLAGS         = '$(strip $(LDFLAGS))')
	$(info COMPILE_MODE    = 'DEBUG')
	$(info unit tests with bounds checking)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) distclean && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) optimizer parser2 ast dag tree scanner3 symbol_table parseinput zonermore robert1 raw_registers asmgen semantic_checks COMPILE_MODE=DEBUG && \
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) clean
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) phase4_leak_tests VALGRIND="$(VALGRIND)" VALGRIND_OPTIONS="$(VALGRIND_OPTIONS)" >phase4_bounds_tests.out 2>&1 || \
	exit 1 && (sort phase4_bounds_tests.out; rm phase4_bounds_tests.out)
	$(MAKE) -f$(ME) $(filter-out -j%,$(MAKEFLAGS)) asmgen_leak_tests VALGRIND="$(VALGRIND)" VALGRIND_OPTIONS="$(VALGRIND_OPTIONS)" >asmgen_bounds_tests.out 2>&1 || \
	exit 1 && (sort asmgen_bounds_tests.out; rm asmgen_bounds_tests.out)

.PHONY:	miscchecks
miscchecks:	distclean ecma55
	./ecma55 -L 2>&1 | cmp -s - COPYING || exit 1

.PHONY:	canrelease
canrelease:	check_with_bounds

.PHONY:	clean
clean:
	$(RM) codegen.o globals.o main.o parser2.o scanner3.o symbol_table.o \
	  dtoa5_normal.o g_fmt_BASIC_normal.o robert1.o parseinput.o peephole.o \
	  zonermore.o tree.o optimizer.o ecma55.1.ps \
	  junk.s ast.o asmgen.o semantic_checks.o error_messages.o raw_registers.o \
	  dag.o load_textdata.o textdata.o textdata.h \
	  textdata.s $(MANPAGES) core asmgen_leak_tests.out phase4_leak_tests.out \
	  phase4_bounds_tests.out asmgen_bounds_tests.out asmgen_tests.out phase4_tests.out \
	  zonermore_test.log symbol_table_test.log raw_registers_test.log scanner3_test.log \
	  scanner3X_test.log parser2_test.log robert1_test.log tree_test.log ast_test.log \
	  optimizer_test.log dag_test.log parseinput_test.log semantic_checks_test.log \
	  asmgen_test*.log

.PHONY:	distclean
distclean:	clean
	$(RM) scanner3 parser2 symbol_table ecma55 dumpregs.o parseinput.s \
	  parseinput peephole zonermore robert1 tree ast optimizer raw_registers \
	  dag asmgen load_textdata globals semantic_checks

%.1.gz:	%.1
	$(GZIP) -9c $^ > $@

zonermore_test:
	@./zonermore >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

symbol_table_test:
	@./symbol_table >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

raw_registers_test:
	@./raw_registers >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

scanner3_test:
	@./scanner3 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

scanner3X_test:
	@./scanner3 -X >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

parser2_test:
	@./parser2 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

robert1_test:
	@./robert1 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  KOWNT=$$(grep -c 0.1130595629080248382125617468 $@.log 2>&1) ;\
	  if ((KOWNT==1)) ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    grep -c 0.1130595629080248382125617468 $@.log 2>&1 ; \
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

tree_test:
	@./tree >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  grep -v '^pointer to' $@.log >$@.log2 ; \
	  if diff -q $@.log2 unit_tests_run_output/$@.log >/dev/null 2>&1 ; \
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log $@.log2 ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log2 unit_tests_run_output/$@.log ; \
	    rm $@.log2 ; \
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

ast_test:
	@./ast >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

optimizer_test:
	@./optimizer >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

dag_test:
	@./dag >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

parseinput_test:
	@printf "NSUNNUS\n1E-9,\"DOG\",123CAT,3.1415926535,-3,1ELEPHANT,\"\"\n" | \
	./parseinput >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

semantic_checks_test:
	@./semantic_checks >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

phase4_tests:	zonermore_test symbol_table_test raw_registers_test scanner3_test scanner3X_test \
	parser2_test robert1_test tree_test ast_test optimizer_test dag_test parseinput_test semantic_checks_test

asmgen_test01:
	@./asmgen >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test02:
	@./asmgen -4 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test04:
	@./asmgen -O1 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test05:
	@./asmgen -O1 -4 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test07:
	@./asmgen -O2 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test08:
	@./asmgen -O2 -4 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test10:
	@./asmgen -O3 -X >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test11:
	@./asmgen -O3 -X -4 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test13:
	@./asmgen -s >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test14:
	@./asmgen -s -4 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test16:
	@./asmgen -s -O1 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test17:
	@./asmgen -s -O1 -4 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test19:
	@./asmgen -s -O2 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test20:
	@./asmgen -s -O2 -4 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test22:
	@./asmgen -s -O3 -X >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_test23:
	@./asmgen -s -O3 -X -4 >$@.log 2>&1 ;\
	RC=$${?} ;\
	if ((RC==0)) ;\
	then \
	  if diff -q $@.log unit_tests_run_output/$@.log >/dev/null 2>&1 ;\
	  then \
	    printf "%-25s PASS\n" $@ ; \
	    rm -f $@.log ; \
	  else \
	    printf "%-25s FAIL (different output)\n" $@ ; \
	    diff -u $@.log unit_tests_run_output/$@.log ;\
	    exit 1; \
	  fi; \
	else \
	  printf "%-25s FAIL (exit code RC=$${RC})\n" $@ ; \
	  exit 1; \
	fi

asmgen_tests:	asmgen_test01 asmgen_test02 asmgen_test04 asmgen_test05 \
	asmgen_test07 asmgen_test08 asmgen_test09 asmgen_test10 asmgen_test11 \
	asmgen_test13 asmgen_test14 asmgen_test16 asmgen_test17 \
	asmgen_test19 asmgen_test20 asmgen_test22 asmgen_test23

zonermore_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./zonermore 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

symbol_table_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./symbol_table 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

raw_registers_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./raw_registers 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

scanner3_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./scanner3 2>&1 | grep -q 'no leaks are possible'&& printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

scanner3X_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./scanner3 -X 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

parser2_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./parser2 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

robert1_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./robert1 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

tree_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./tree 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

ast_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./ast 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

optimizer_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./optimizer 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

dag_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./dag 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

parseinput_leak_test:
	@printf "NSUNNUS\n1E-9,\"DOG\",123CAT,3.1415926535,-3,1ELEPHANT,\"\"\n" |  $(VALGRIND) $(VALGRIND_OPTIONS) ./parseinput 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

semantic_checks_leak_test:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./semantic_checks 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

phase4_leak_tests:	zonermore_leak_test symbol_table_leak_test raw_registers_leak_test scanner3_leak_test \
	scanner3X_leak_test parser2_leak_test robert1_leak_test tree_leak_test ast_leak_test optimizer_leak_test \
	dag_leak_test parseinput_leak_test semantic_checks_leak_test

asmgen_leak_test01:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test02:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -4 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test04:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -O1 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test05:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -O1 -4 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test07:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -O2 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test08:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -O2 -4 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test10:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -O3 -X 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test11:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -O3 -X -4 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test13:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -s 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test14:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -s -4 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test16:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -s -O1 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test17:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -s -O1 -4 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test19:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -s -O2 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test20:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -s -O2 -4 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test22:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -s -O3 -X 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_test23:
	@$(VALGRIND) $(VALGRIND_OPTIONS) ./asmgen -s -O3 -X -4 2>&1 | grep -q 'no leaks are possible' && printf "%-25s PASS\n" $@ || (printf "%-25s FAIL\n" $@ ; exit 1;)

asmgen_leak_tests:	asmgen_leak_test01 asmgen_leak_test02 asmgen_leak_test04 \
	asmgen_leak_test05 asmgen_leak_test07 asmgen_leak_test08 asmgen_leak_test09 \
	asmgen_leak_test10 asmgen_leak_test11 asmgen_leak_test13 asmgen_leak_test14 \
	asmgen_leak_test16 asmgen_leak_test17 asmgen_leak_test19 \
	asmgen_leak_test20 asmgen_leak_test22 asmgen_leak_test23

.PHONY:	asmgen_tests asmgen_test01 asmgen_test02 asmgen_test04 asmgen_test05 \
	asmgen_test07 asmgen_test08 asmgen_test09 asmgen_test10 asmgen_test11 \
	asmgen_test13 asmgen_test14 asmgen_test16 asmgen_test17 \
	asmgen_test19 asmgen_test20 asmgen_test22 asmgen_test23 \
	emsg asmgen_leak_tests emsg \
	asmgen_leak_test01 asmgen_leak_test02 asmgen_leak_test04 \
	asmgen_leak_test05 asmgen_leak_test07 asmgen_leak_test08 asmgen_leak_test09 \
	asmgen_leak_test10 asmgen_leak_test11 asmgen_leak_test13 asmgen_leak_test14 \
	asmgen_leak_test16 asmgen_leak_test17 asmgen_leak_test19 \
	asmgen_leak_test20 asmgen_leak_test22 asmgen_leak_test23 \
	phase4_tests zonermore_test symbol_table_test raw_registers_test scanner3_test scanner3X_test parser2_test \
	robert1_test tree_test ast_test optimizer_test dag_test parseinput_test semantic_checks_test \
	phase4_leak_tests zonermore_leak_test symbol_table_leak_test raw_registers_leak_test scanner3_leak_test \
	scanner3X_leak_test parser2_leak_test robert1_leak_test tree_leak_test ast_leak_test optimizer_leak_test \
	dag_leak_test parseinput_leak_test semantic_checks_leak_test

