//
// error_messages.c - This is the file containing error messages
//                    for Mr. Ham's ECMA-55 Minimal BASIC compiler.
//
// Copyright (C) 2015,2016,2017,2018,2019,2020,2021  John Gatewood Ham
//
// This file is part of ecma55.
//
// ecma55 is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2
// as published by the Free Software Foundation.
//
// ecma55 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ecma55.  If not, see <http://www.gnu.org/licenses/>.
//
#include <inttypes.h>
#include <string.h>
#include "error_messages.h"

char const * const emsg[] = {
/*  0 */   "ERROR: Invalid line number %s; line numbers cannot be negative\n",
/*  1 */   "ERROR: Invalid line number %s; must be between 1 and %" PRIu32 "\n",
/*  2 */   "ERROR: Invalid line number %s; cannot convert to unsigned long\n",
/*  3 */   "ERROR: Invalid line number %s; trailing garbage is '%s'(scanner failure?)\n",
/*  4 */   "ERROR: Invalid line number %s; must be between 1 and %" PRIu32 "\n",
/*  5 */   "ERROR: attempting to allocate already allocated RAM!\n",
/*  6 */   "ERROR: overflow of the ramtastic table - incrase MAXTRACKED\n",
/*  7 */   "ERROR: %s() attempted on previously freed RAM\n",
/*  8 */   "ERROR: %s() attempted on untracked RAM\n",
/*  9 */   "ERROR: %s() attempted to change untracked RAM\n",
/* 10 */   "ERROR: %s() passed a NULL pointer\n",
/* 11 */   "------------UNUSED",
/* 12 */   "------------UNUSED",
/* 13 */   "ERROR: %s() called with numchildren > 2\n",
/* 14 */   "ERROR: Unsupported token %s encountered during pretty print!\n",
/* 15 */   "ERROR: %s():unhandled oid for %s\n",
/* 16 */   "ERROR: Bad power on line number %" PRIu32 "\n",
/* 17 */   "ERROR: %s():bad register conversion for '%s' with errno=%d and tuival=%" PRIu32 "\n",
/* 18 */   "ERROR: %s():bad register conversion for s='%s' with errno=%d and tuival=%" PRIu32 ", junk at end of string\n",
/* 19 */   "------------UNUSED",
/* 20 */   "ERROR: %s(): pop on empty xmmregs stack\n",
/* 21 */   "ERROR: %s(): push on full xmmregs stack, tos=%" PRIu8 ",stack_max=%" PRIu8 "\n",
/* 22 */   "------------UNUSED",
/* 23 */   "------------UNUSED",
/* 24 */   "ERROR: No array data found for array variable '%s'\n",
/* 25 */   "ERROR: %s() on line %" PRIu32 " write to array %s using %" PRIu32 " dimensions is not supported.\n",
/* 26 */   "ERROR: %s() cannot add float literal for %s to symbol table\n",
/* 27 */   "------------UNUSED",
/* 28 */   "------------UNUSED",
/* 29 */   "------------UNUSED",
/* 30 */   "ERROR: Out of memory in %s()\n",
/* 31 */   "------------UNUSED",
/* 32 */   "------------UNUSED",
/* 33 */   "------------UNUSED",
/* 34 */   "ERROR: %s(): more than %" PRIu32 " jump targets specified\n",
/* 35 */   "ERROR: Semantic check failed.\n",
/* 36 */   "ERROR: On line %" PRIu32 " read of array %s is using %" PRIu32 " dimensions, but it actually has %" PRIu32 " dimensions\n",
/* 37 */   "ERROR: On line %" PRIu32 " read of array %s is using %" PRIu32 " dimensions is not supported.\n",
/* 38 */   "ERROR: Bad LHS in %s statement\n",
/* 39 */   "ERROR: Impossible number of dimensions in %s()\n",
/* 40 */   "ERROR: Catestrophic symbol table failure while processing %s of variable %s on line %" PRIu32 "\n",
/* 41 */   "ERROR: Unable to allocate %" PRIu64 " bytes of memory in %s()\n",
/* 42 */   "ERROR: Unsupported token %s encountered during pretty print!\n",
/* 43 */   "ERROR: Statement beginning with %s is not supported.\n",
/* 44 */   "ERROR: Unknown type of DATA\n",
/* 45 */   "ERROR: NULL pointer or empty value for source of floating point literal\n",
/* 46 */   "ERROR: Invalid pointer for generated name of floating point literal\n",
/* 47 */   "------------UNUSED",
/* 48 */   "ERROR: NULL pointer for user-defined function name on line %" PRIu32 "\n",
/* 49 */   "ERROR: User-defined function name cannot be an empty string on line %" PRIu32 "\n",
/* 50 */   "ERROR: Invalid function name '%s'\n",
/* 51 */   "ERROR: NULL pointer for source of string literal\n",
/* 52 */   "ERROR: Invalid pointer for destination of string literal\n",
/* 53 */   "------------UNUSED",
/* 54 */   "ERROR: NULL pointer passed to %s()\n",
/* 55 */   "ERROR: Empty string passed to %s()\n",
/* 56 */   "ERROR: Invalid numeric array variable name (does not begin with capital letter) specified to %s()\n",
/* 57 */   "ERROR: Invalid numeric array variable name (more than 1 byte) specified to %s()\n",
/* 58 */   "------------UNUSED",
/* 59 */   "ERROR: Variable name '%s' not found in symbol table on line %" PRIu32 "\n",
/* 60 */   "ERROR: %s(): expected %s but got %s on line %" PRIu32 "\n",
/* 61 */   "ERROR: Unknown type in %s()\n",
/* 62 */   "ERROR: %s() called with invalid tid\n",
/* 63 */   "------------UNUSED",
/* 64 */   "------------UNUSED",
/* 65 */   "ERROR: %s() called but not handled correctly\n",
/* 66 */   "------------UNUSED",
/* 67 */   "------------UNUSED",
/* 68 */   "ERROR: No string expression after = for assignment to %s on line %" PRIu32 "\n",
/* 69 */   "ERROR: %s():cannot handle %s\n",
/* 70 */   "------------UNUSED",
/* 71 */   "------------UNUSED",
/* 72 */   "------------UNUSED",
/* 73 */   "ERROR: Impossible array dimensions encountered in %s()\n",
/* 74 */   "ERROR: Impossible token type encountered in %s()\n",
/* 75 */   "ERROR: Invalid number %s; unsigned integers cannot be negative.\n",
/* 76 */   "ERROR: On line %" PRIu32 " write to array %s using %" PRIu32 " dimensions is not supported.\n",
/* 77 */   "------------UNUSED",
/* 78 */   "ERROR: %s code generation failed.\n",
/* 79 */   "ERROR: Unkown variable type for %s in %s statement on line %" PRIu32 "\n",
/* 80 */   "ERROR: Bad array read\n",
/* 81 */   "ERROR: Unknown variable type %s for %s in %s statement on line %" PRIu32 "\n",
/* 82 */   "ERROR: %s() encountered unknown tid %s\n",
/* 83 */   "ERROR: Unknown relational operator\n",
/* 84 */   "ERROR: Unknown statement.\n",
/* 85 */   "ERROR: Unable to open file '%s for output.\n",
/* 86 */   "ERROR: Unable to write data segment to file '%s'.\n",
/* 87 */   "ERROR: %s(): %s parameter is NULL\n",
/* 88 */   "ERROR: %s(): %s parameter is NULL or empty\n",
/* 89 */   "ERROR: %s(): lineno %" PRIu32 " is greater than %" PRIu32 "\n",
/* 90 */   "------------UNUSED",
/* 91 */   "------------UNUSED",
/* 92 */   "ERROR: Scanner failed on %" PRIu32 "%s line of input\n",
/* 93 */   "ERROR: Scanner failed on %" PRIu32 "%s line of input (line %lu of scanner3.c)\n",
/* 94 */   "ERROR: Jumping from %" PRIu32 " into loop at %" PRIu32 " not permitted\n",
/* 95 */   "ERROR: Duplicate line number %" PRIu32 "\n",
/* 96 */   "ERROR: Line number %" PRIu32 " is less than previous line number %" PRIu32 "\n",
/* 97 */   "ERROR: On line number %" PRIu32 ", '%s' is not valid in strict ECMA-55 mode.  Maybe you need the -X option?\n",
/* 98 */   "ERROR: On line %" PRIu32 " attempted to reference user-defined function '%s' inside its own definition.\n",
/* 99 */   "ERROR: On line %" PRIu32 " attempt to invoke undefined user-defined function named %s\n",
/* 100 */  "ERROR: On line %" PRIu32 " function '%s' called without an argument but defined with one.\n",
/* 101 */  "ERROR: On line %" PRIu32 " %s %s without corresponding %s\n",
/* 102 */  "ERROR: On line %" PRIu32 " %s %s but expecting %s %s\n",
/* 103 */  "ERROR: On line %" PRIu32 " %s %s at scope %s but expecting scope %s\n",
/* 104 */  "ERROR: On line %" PRIu32 " %s %s but scope is global\n",
/* 105 */  "ERROR: On line %" PRIu32 " unable to use %s as a loop index variable in FOR\n",
/* 106 */  "ERROR: On line %" PRIu32 " variable %s in FOR is a control variable in an\nouter scope FOR statement on line %" PRIu32 "\n",
/* 107 */  "ERROR: FOR stack depth of %" PRIu32 " exceeded.\n",
/* 108 */  "ERROR: Unable to use %s as a loop increment variable\n",
/* 109 */  "ERROR: Unable to use %s as a loop limit variable\n",
/* 110 */  "ERROR: on line %" PRIu32 " assignment to array %s is using %" PRIu32 " dimensions, but it actually has %" PRIu32 " dimensions\n",
/* 111 */  "ERROR: On line %" PRIu32 " unable to create array of name '%s'\n",
/* 112 */  "ERROR: On line %" PRIu32 " %s specified an upper bound of zero but the minimum has been set\nto 1 by the %s %s statement\n",
/* 113 */  "ERROR: On line %" PRIu32 " function '%s' called with an argument but defined without one.\n",
/* 114 */  "ERROR: On line %" PRIu32 " %s for variable %s failed since it already has been dimensioned\n       with %" PRIu32 " dimensions, dim1=%" PRIu32 "\n",
/* 115 */  "ERROR: On line %" PRIu32 " %s for variable %s failed since it already has been dimensioned\n       with %" PRIu32 " dimensions, dim1=%" PRIu32 ", dim2=%" PRIu32 "\n",
/* 116 */  "ERROR: On line %" PRIu32 " %s for variable %s failed but I do not know why\n",
/* 117 */  "ERROR: Array with %" PRIu32 " dimensions should not be possible.\n",
/* 118 */  "ERROR: On line %" PRIu32 " invalid value for OPTION BASE; must be 0 or 1\n",
/* 119 */  "ERROR: On line %" PRIu32 " invalid use of OPTION BASE statement\n",
/* 120 */  "ERROR: Missing final END statement.\n",
/* 121 */  "ERROR: On line %" PRIu32 " %s %s statement without %s\n",
/* 122 */  "ERROR: On line %" PRIu32 " program contains a READ statement but program has no DATA statements.\n",
/* 123 */  "ERROR: On line %s an END statement was found, but that is not the last line of the program.\n",
/* 124 */  "ERROR: On line %" PRIu32 " input line too long; maximum permitted line length is %" PRIu32 " including newline\n",
/* 125 */  "       character is %s (%d)\n",
/* 126 */  "ERROR: Minimal BASIC does not permit character '%s' (%u) here.\n",
/* 127 */  "ERROR: Minimal BASIC does not permit character '%c' (%u) here.\n",
/* 128 */  "ERROR: reject on %" PRIu32 "%s column of %" PRIu32 "%s line\n",
/* 129 */  "ERROR: Minimal BASIC does not permit lower-case letters.\n",
/* 130 */  "ERROR: ECMA-55 does not permit leading or trailing spaces in an unquoted string\nin a DATA statement.\n",
/* 131 */  "ERROR: This error is on or near the BASIC input program line number %" PRIu32 "\n",
/* 132 */  "ERROR: This error is on the very first line of the BASIC input program and\na line number is required.\n",
/* 133 */  "       character is a newline (%d)\n",
/* 134 */  "       character is '%c' (%d)\n",
/* 135 */  "ERROR: Unknown type when generating data segment\n",
/* 136 */  "ERROR: Invalid user-defined function name '%s' (not exactly 3 bytes) specified to add_udf()\n",
/* 137 */  "ERROR: Invalid user-defined function name '%s' (does not begin with 'FN') specified to add_udf()\n",
/* 138 */  "ERROR: Invalid user-defined function name '%s' (third character is not a capital letter) specified to add_udf()\n",
/* 139 */  "ERROR: user-defined function '%s' on line %" PRIu32 " has already been defined.\n",
/* 140 */  "ERROR: Invalid numeric scalar variable name '%s' (does not begin with capital letter) specified to nvar_exists()\n",
/* 141 */  "ERROR: Invalid numeric scalar variable name '%s' (second byte is not a digit) specified to nvar_exists()\n",
/* 142 */  "ERROR: Invalid numeric scalar variable name '%s' (more than 2 bytes) specified to nvar_exists()\n",
/* 143 */  "ERROR: Invalid numeric array variable name '%s' (does not begin with capital letter) specified to navar_exists()\n",
/* 144 */  "ERROR: Invalid numeric array variable name '%s' (more than 1 byte) specified to navar_exists()\n",
/* 145 */  "ERROR: Invalid user-defined function name '%s' (not exactly 3 bytes) specified to udf_exists()\n",
/* 146 */  "ERROR: Invalid user-defined function name '%s' (does not begin with 'FN') specified to udf_exists()\n",
/* 147 */  "ERROR: Invalid user-defined function name '%s' (third character is not a capital letter) specified to udf_exists()\n",
/* 148 */  "ERROR: Invalid string scalar variable name '%s' (does not begin with capital letter) specified to svar_exists()\n",
/* 149 */  "ERROR: Invalid string scalar variable name '%s' (second byte is not a dollar sign) specified to svar_exists()\n",
/* 150 */  "ERROR: Invalid string scalar variable name '%s' (more than 2 bytes) specified to svar_exists()\n",
/* 151 */  "ERROR: Array variable '%s' cannot be assigned a numeric scalar value on line %" PRIu32 "\n",
/* 152 */  "ERROR: Scalar variable '%s' cannot be assigned a numeric array value on line %" PRIu32 "\n",
/* 153 */  "ERROR: Number of dimensions must be 1 or 2\n",
/* 154 */  "ERROR: Invalid array name '%s'\n",
/* 155 */  "ERROR: specified a non-zero maximum subscript %" PRIu32 " for 2nd dimension in vector '%s'\n",
/* 156 */  "ERROR: Unable to write data segment to file '%s'.\n",
/* 157 */  "ERROR: Unable to open file '%s for output.\n",
/* 158 */  "------------UNUSED",
/* 159 */  "ERROR: Invalid number %s; must be between 0 and %" PRIu32 " inclusive\n",
/* 160 */  "ERROR: Invalid number %s; cannot convert to uint32_t\n",
/* 161 */  "ERROR: Invalid number %s; trailing garbage is '%s' (scanner failure?)\n",
/* 162 */  "ERROR on line %" PRIu32 ": Expected %s, but got %s",
/* 163 */  " with value '%s'",
/* 164 */  " instead.\n",
/* 165 */  "ERROR: On line %" PRIu32 ", function '%s' called with an empty argument list.\n",
/* 166 */  "ERROR: line number '%s' may not contain leading whitespace\n",
/* 167 */  "ERROR: Invalid line number %" PRIu32 "; minimum line number is 1\n",
/* 168 */  "ERROR: Invalid line number %" PRIu32 "; maximum line number is %" PRIu32 "\n",
/* 169 */  "ERROR: %s on line %" PRIu32 "\n",
/* 170 */  "ERROR: %s on line %" PRIu32 " specified %" PRIu32 " elements; maximum permitted is %" PRIu32 "\n",
/* 171 */  "ERROR on line %" PRIu32 ": Expected an arithmetic function, a numeric scalar\nor array variable, "
           "a numeric constant, or a left parenthesis,\nbut got token id %s, text='%s'\n",
/* 172 */  "ERROR on line %" PRIu32 ": string expressions do not permit <= operator\n",
/* 173 */  "ERROR on line %" PRIu32 ": string expressions do not permit < operator\n",
/* 174 */  "ERROR on line %" PRIu32 ": string expressions do not permit >= operator\n",
/* 175 */  "ERROR on line %" PRIu32 ": string expressions do not permit > operator\n",
/* 176 */  "ERROR on line %" PRIu32 ": unknown relational operator %s\n",
/* 177 */  "ERROR on line %" PRIu32 ": expected string expression on right hand side of relational expression\n",
/* 178 */  "ERROR on line %" PRIu32 ": expected numeric expression on right hand side of relational expression\n",
/* 179 */  "ERROR: Missing destination variable on line %" PRIu32 "\n",
/* 180 */  "ERROR: Invalid variable name in %s statement on line %" PRIu32 "\n",
/* 181 */  "ERROR: unquoted string datum on line %" PRIu32 " is too long.\n"
           "       Maximum permitted length of an unquoted string datum is %" PRIu32 " bytes\n",
/* 182 */  "ERROR: quoted string datum on line %" PRIu32 " is too long.\n"
           "       Maximum permitted length of a quoted string datum is %" PRIu32 " bytes\n",
/* 183 */  "ERROR: Empty datum in %s statement on line %" PRIu32 "\n",
/* 184 */  "ERROR on line %" PRIu32 ": Expected %s, but got %s with value '%s' instead\n",
/* 185 */  "ERROR: On line %" PRIu32 ", invalid text after %s keyword: '%s'.\n"
           "       Expected numeric expression, string literal, or a string variable.\n",
/* 186 */  "ERROR: Numeric variable '%s' is a scalar, not an array on line %" PRIu32 "\n",
/* 187 */  "ERROR: Bad %s statement on line %u: got %s but expected numeric or string expression.\n",
/* 188 */  "ERROR on line %" PRIu32 ": OPTION BASE requires an argument of 0 or 1\n",
/* 189 */  "ERROR on line %" PRIu32 ": missing %s keyword in assignment\n",
/* 190 */  "ERROR: END statement on line %" PRIu32 " must be the last statement in the program.\n",
/* 191 */  "ERROR: END statement must be the last statement in the program.\n",
/* 192 */  "ERROR: Missing line number\n",
/* 193 */  "ERROR: Expected line '10', got line '%s'\n",
/* 194 */  "ERROR: parseit() failed\n",
/* 195 */  "------------UNUSED",
/* 196 */  "ERROR: Self-test failed\n",
/* 197 */  "ERROR on line %" PRIu32 ": Expected a valid keyword after the line number but did not find one.\n",
/* 198 */  "Valid keywords are:",
/* 199 */  "ERROR: You tried an unimplemented unary operator\n",
/* 200 */  "ERROR: You tried an unimplemented binary operator\n",
/* 201 */  "ERROR: Logical operator '%s' not supported.\n",
/* 202 */  "ERROR: '%s' is not an implemented unary operator\n",
/* 203 */  "ERROR: You tried an unimplemented form of minus operator\n",
/* 204 */  "ERROR: Unknown parent binary operator encountered '%s'\n",
/* 205 */  "ERROR: Unknown child binary operator encountered '%s'\n",
/* 206 */  "ERROR: Crash determining READ target variable type.\n",
/* 207 */  "ERROR: Crash determining INPUT target variable type.\n",
/* 208 */  "------------UNUSED",
/* 209 */  "ERROR: Attempt to deallocate non-allocated register '%s'\n",
/* 210 */  "ERROR: node in tree passed to tree_delete_all() has NULL leaftoken....\n",
/* 211 */  "ERROR: Statement beginning with %s is not supported yet.\n",
/* 212 */  "ERROR: dlopen failed: %s\n",
/* 213 */  "ERROR: dlsym failed for '%s': %s\n",
/* 214 */  "------------UNUSED",
/* 215 */  "ERROR: ECMA-55 Minimal BASIC only supports 7-bit ASCII\n",
/* 216 */  "ERROR: Missing line number!\n       Line number must begin in first column.\n",
/* 217 */  "ERROR: On line %" PRIu32 ", unknown keyword '%s'.\n",
/* 218 */  "ERROR: On line %" PRIu32 " missing space or '(' before keyword '%s'.\n",
/* 219 */  "ERROR: On line %" PRIu32 " missing space before keyword '%s'.\n",
/* 220 */  "ERROR: On line %" PRIu32 " missing space after keyword '%s'.\n",
/* 221 */  "ERROR: On line %" PRIu32", %s\n",
/* 222 */  "ERROR: On line %" PRIu32 " extensions keyword '%s' was used.\n"
           "       ECMA-55 Minimal BASIC does not support that keyword.\n"
           "       Maybe you need the '-X' option?\n",
};
